#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/dir.h>
#include <ctype.h>
#include <locale.h>
#include <sys/wait.h>
#include <sys/time.h>	
#include <fcntl.h>
#include <grp.h>
#include <termcap.h>
#include <wchar.h>
#include <dirent.h>
#define direct dirent
#include "dir.h"
#include "ice.h"
#define QUANT 128
#define ISDIGIT(c) ((c) >= '0' && (c) <= '9')
//#define S_ISLNK( mode )   (((mode) & S_IFMT) == S_IFLNK)
#include <syslog.h>

int showhidden = 1;
int uid;                                /* real user id */
int gidnum;                             /* number of groups in list */

int gidlist [128];              /* groups list */

static struct dir *compdir;

int comptype (const struct file *f);

static char *prepend (char *p, char *str);

int H;

#define STOREINT(p,nn) {\
	register char *cp = p;\
	register int n = nn;\
	*cp++ = n/10%10 + '0';\
	*cp = n%10 + '0';\
	}

static char pattern [] = "00-Mon-1900 00:00:00";

char *timestr (long tim)
{
	struct tm *t = localtime (&tim);
	register char *p = pattern;
	STOREINT (p, t->tm_mday);
	if (*p == '0')
		*p = ' ';


	strncpy (p+3, "JanFebMarAprMayJunJulAugSepOctNovDec"+t->tm_mon*3, 3);
	STOREINT (p+9, t->tm_year);
	if (t->tm_year < 100) {
		p[7] = '1';
		p[8] = '9';
	} else {
		p[7] = '2';
		p[8] = '0';
	}
	STOREINT (p+12, t->tm_hour);
	STOREINT (p+15, t->tm_min);
	STOREINT (p+18, t->tm_sec);
	return (p);
}

int mygroup (int gid)
{
	int i;

	for (i=0; i<gidnum; ++i)
		if (gid == gidlist[i]) 
			return 1;
	return 0;
}

char *mdup (char *s)
{
	char *p;

	p = malloc (strlen (s) + 1);
	strcpy (p, s);
	return (p);
}


int match2 (char *name, char *pat)
{
	int ok, negate_range, matched;

	if (*pat == '^') {
		matched = 0;
		++pat;
	} else
		matched = 1;
	for (;;) {
		switch (*pat++) {
		case '*':
			if (! *pat)
				return (matched);
			for (; *name; ++name)
				if (match2 (name, pat))
					return (matched);
			return (! matched);
		case '?':
			if (! *name++)
				return (! matched);
			break;
		case 0:
			if (*name)
				return (! matched);
			return (matched);
		default:
			if (*name++ != pat[-1])
				return (! matched);
			break;
		case '[':
			ok = 0;
			negate_range = (*pat == '^' || *pat == '!');
			if (negate_range)
				++pat;
			while (*pat++ != ']')
				if (*pat == '-') {
					if (pat[-1] <= *name && *name <= pat[1])
						ok = 1;
					pat += 2;
				} else if (pat[-1] == *name)
					ok = 1;
			if (ok == negate_range)
				return (! matched);
			++name;
			break;
		}
	}
}

static int fgetinfo (char *info, int maxlen, char *rinfo, int rmaxlen,
	unsigned long *date, FILE *fd)
{
	char *p;
	int n;

	*info = *rinfo = 0;
	*date = 0;
again:
	do n = getc (fd);
	while (n == ' ' || n == '\t');
	if (n < 0)
		return (0);
	ungetc (n, fd);

	fgets (info, maxlen, fd);
	if (ISDIGIT (info[0]) && ISDIGIT (info[1]) &&
	    ISDIGIT (info[2]) && ISDIGIT (info[3]) &&
	    ISDIGIT (info[4]) && ISDIGIT (info[5])) {
		char *q = info;

		*date = strtol (info, 0, 0);
		for (p=info+6; *p==' ' || *p=='\t'; ++p)
			continue;
		while (*p)
			*q++ = *p++;
		*q = 0;
	}
	p = info;
	while (maxlen > 0) {
		n = strlen (p);
		maxlen -= n;
		p += n;

		while (--n >= 0 && (p[-1] == '\n' || p[-1] == '\15')) {
			*--p = 0;
			++maxlen;
		}

		n = getc (fd);
		if (n < 0)
			break;
		if (n != ' ' && n != '\t') {
			char buf[9];

			ungetc (n, fd);
			if (n != 'R' || ! fgets (buf, 9, fd) ||
			    strncmp (buf, "Russian:", 8) != 0)
				break;
			info = rinfo;
			maxlen = rmaxlen;
			goto again;
		}

		do n = getc (fd);
		while (n == ' ' || n == '\t');
		if (n < 0)
			break;
		ungetc (n, fd);
		*p++ = ' ';
		--maxlen;

		fgets (p, maxlen, fd);
	}
	return (1);
}

void setinfo (struct file *f, struct dir *d)
{
	char info[512], rinfo[512];
	FILE *fd;

	f->infoflag = 1;
	f->infodate = 0;

	/* No info for . and .. */
	if (f->name[0] == '.' && (f->name[1] == 0 ||
	   (f->name[1] == '.' && f->name[2] == 0)))
		return;

	sprintf (info, "%s/.info/%s", d->cwd, f->name);
	fd = fopen (info, "r");
	if (fd) {
		fgetinfo (info, sizeof (info), rinfo, sizeof (rinfo),
			&f->infodate, fd);
		if (*info) {
			if (strlen (info) > 45)
				info[45] = 0;
			f->info = mdup (info);
		}
		if (*rinfo) {
			if (strlen (rinfo) > 45)
				rinfo[45] = 0;
			f->rinfo = mdup (rinfo);
		}
		fclose (fd);
	}
	if (f->infodate)
		f->dateflag = 1;
	else {
		struct tm *t = localtime (&f->mtime);

		f->infodate = t->tm_year % 100 * 10000 + 
			(t->tm_mon + 1) * 100 + t->tm_mday;
		f->dateflag = 0;
	}
}

int compfile (const void *arg1, const void *arg2)
{
	const struct file *a = arg1;
	const struct file *b = arg2;
	int atype, btype;
	int rez;

	if(strcmp((char *)a,"..")==0) return 0;

	if (compdir->revsort) {
		const struct file *t;

		t = a;
		a = b;
		b = t;
	}
	if (compdir->typesort) {
		atype = comptype (a);
		btype = comptype (b);
		if (atype != btype)
		{
			return (atype<btype ? -1 : 1);
		}
	}
	switch (compdir->sort) 
	{
	case SORTEXT:
		atype = a->name[0] != '.';
		btype = b->name[0] != '.';
		if (atype != btype)
		{
			return (atype<btype ? -1 : 1);
		}
		if ((rez = strcmp (extension (a->name), extension (b->name))))
		{
			return (rez);
		}
	case SORTNAME:
		return (strcmp (a->name, b->name));
	case SORTTIME:
		return (a->mtime < b->mtime ? -1 :
			a->mtime > b->mtime ? 1 : 0);
	case SORTSIZE:
		return (a->size < b->size ? -1 :
			a->size > b->size ? 1 : 0);
	}
	return (0);
}

/*
 *      Return integer number, used for comparing file types.
 */
int comptype (const struct file *f)
{
	switch (f->mode & S_IFMT) {
	case S_IFDIR:
		if (f->name[0] == '.') {
			if (! f->name[1])
				return (-1);
			if (f->name[1] == '.' && ! f->name[2])
				return (-2);
		}
		if (! f->execable)
			return (1);
		return (0);
	case S_IFCHR:
		return (100);
	case S_IFBLK:
		return (110);
#ifdef S_IFIFO
	case S_IFIFO:
		return (120);
#endif
#ifdef S_IFNAM
	case S_IFNAM:
		return (130);
#endif
#ifdef S_IFSOCK
	case S_IFSOCK:
		return (140);
#endif
#ifdef S_IFLNK
	case S_IFLNK:
		return (150);
#endif
	case S_IFREG:
		return (500);
	default:
		return (900);
	}
}

void chdirup (char *name)
{
	register char *s, *p;


	for (;;) {
		if (chdir (name) < 0)
		{
			//error ("Cannot chdir to %s", name);
		}
		else if (access (".", 4) < 0)
		{
			//error ("Cannot read %s", name);
		}
		else
			break;
		if (name[0]=='/' && name[1]==0) {       /* root directory */
		}


		/* cut last directory name */
		for (p=s=name; *s; ++s)
			if (*s == '/')
				p = s;
		*p = 0;
		if (! *name) {
			name [0] = '/';
			name [1] = 0;
		}
	}
}

static char *prepend (char *p, char *str)
{
	int len = strlen (str);
	memcpy (p-len, str, len);
	return (p-len);
}

/*
 * Get current directory name.  Pathname should be at least
 * MAXPATHLEN long.  Returns 0 on error, and puts message into pathname.
 * Compatible with BSD getwd().
 */
char *getcwdname (char *pathname)
{
	char upname [MAXPATHLEN+MAXNAMLEN];
	char myname [MAXPATHLEN];
	char *upptr, *myptr;
	struct stat root, up, cur, f;
	register struct direct *rec;
	DIR *dir;

	myptr = &myname[MAXPATHLEN-1];
	upptr = &upname[MAXPATHLEN-1];
	*myptr = *upptr = 0;
	upptr = prepend (upptr, "./");
	stat ("/", &root);
	stat (upptr, &cur);
	while (cur.st_dev != root.st_dev || cur.st_ino != root.st_ino) {
		upptr = prepend (upptr, "../");
		dir = opendir (upptr);
		if (! dir) {
			strcpy (pathname, "Cannot read `..'");
			return (0);
		}
		fstat (dirfd (dir), &up);

		int ii=0;
		for (;;) 
		{
			if (! (rec = readdir (dir))) {
				strcpy (pathname, "Cannot find path");
				return (0);
			}
		
			ii++;

			if (cur.st_dev == up.st_dev) {
				if (cur.st_ino == rec->d_ino)
					break;
				continue;
			}
			if (rec->d_name[0]=='.' && (!rec->d_name[1] ||
			   (rec->d_name[1]=='.' && rec->d_name[2]==0)))
				continue;
			strcpy (&upname[MAXPATHLEN-1], rec->d_name);
			lstat (upptr, &f);
			upname[MAXPATHLEN-1] = 0;
			if (cur.st_ino == f.st_ino && cur.st_dev == f.st_dev)
				break;
		}
		myptr = prepend (myptr, rec->d_name);
		*--myptr = '/';
		closedir (dir);
		cur = up;
	}
	strcpy (pathname, *myptr ? myptr : "/");
	return (pathname);
}

void counttag (struct dir *d)
{
	register struct file *f;

	/* compute number of files and bytes */
	d->tfiles = 0;
	d->tbytes = 0;
	for (f=d->cat; f<d->cat+d->num; ++f)
		if ((f->mode & S_IFMT) == (unsigned) S_IFREG && f->tag) 
		{
			++d->tfiles;
			d->tbytes += f->size;
		}
}


void retag (struct dir *d, struct dir *od)
{
	register struct file *f, *of;
	struct file *end, *oend, *ocur, *otop;
	int cmp;

	H=LINES/2-5;

	end = d->cat + d->num;
	oend = od->cat + od->num;
	ocur = od->cat + od->curfile;
	otop = od->cat + od->topfile;
	d->curfile = -1;
	compdir = d;
	for (f=d->cat, of=od->cat; of<oend; ++of) {
		while ((cmp = compfile (f, of)) < 0)
			if (++f >= end)
				goto breakloop;
		if (of == ocur)
			d->curfile = f - d->cat;
		if (of == otop)
			d->topfile = f - d->cat;
		if (cmp == 0)
			f->tag = of->tag;
	}
breakloop:
	if (d->curfile < 0) {
		d->curfile = d->num - 1;
		d->topfile = d->curfile - height() + 1;
	} else if (d->topfile > d->curfile || d->topfile + height() <= d->curfile)
		d->topfile = d->curfile - height() + 1;
	while (d->topfile + height() - H > d->curfile)
		d->topfile -= H;
	if (d->topfile < 0)
		d->topfile = 0;
}




/*
 *      Set up new directory.
 *      struct dir *d   - directory structure
 *      char *dirname   - name of directory.
 *                      If 0 then use d->cwd.
 */
int setdir (struct dir *d, char *dirname)
{
	DIR *f;
	struct direct *recp;
	register struct file *p;
	struct dir olddir;
	struct stat st;

//	if (dirname)
	if (chdir (dirname) < 0)
			{
				return -1;
			}
		else
			getcwdname (d->cwd);
	chdirup (d->cwd);
	d->shortcwd = strtail (d->cwd, '/', 35);
	stat (d->cwd, &st);
	d->dev = st.st_dev;
	d->ino = st.st_ino;

	f = opendir (".");
	if (! f) 
	{
		//		error ("Cannot read '.' -- good bye !");
		//		exit (0);
	}


	if (! dirname) {
		/* save directory structure, then use it for retagging */
		olddir = *d;
		d->catsz = 0;
		d->cat = 0;
	}

	if (! d->cat) {
		d->catsz = QUANT;
		d->cat = (struct file*) calloc (d->catsz, sizeof (struct file));
	} else
		for (p=d->cat; p<d->cat+d->num; ++p) {
			if (p->info)
				free (p->info);
			if (p->rinfo)
				free (p->rinfo);
			p->info = p->rinfo = 0;
		}


	d->num = 0;
	d->nfiles = 0;
	d->tfiles = 0;
	d->nbytes = 0;
	d->mfiles = 0;
	d->mbytes = 0;
	while ((recp = readdir (f))) 
	{

		if (recp->d_name[0] == '.' && recp->d_name[1] == 0 ) continue; // "." ->  


		if (! recp->d_ino)
			continue;
		mcheck (d->cat, struct file*, d->catsz, QUANT, d->num);
		p = &d->cat[d->num];
		lstat (recp->d_name, &st);
		p->link = ((st.st_mode & S_IFMT) == (unsigned) S_IFLNK);

		if (p->link)
		{
			stat (recp->d_name, &st);
		}
		/* skip .. in root directory */
		if (recp->d_name[0] == '.' && recp->d_name[1] == '.' &&
		    recp->d_name[2] == 0 && st.st_ino == d->ino &&
		    st.st_dev == d->dev)
			continue;

		/* skip all hidden files except parent dir entry */
		if (! showhidden && recp->d_name[0] == '.' &&
		    (recp->d_name[1] != '.' || recp->d_name[2]))
			continue;

		/* skip names not matched with pattern */
		if ((st.st_mode & S_IFMT) == (unsigned) S_IFREG) {
			if (! showhidden && recp->d_name[0] == '.')
				continue;
			++d->nfiles;
			d->nbytes += st.st_size;
			if (d->pattern [0] && ! match2 (recp->d_name, d->pattern))
				continue;
			++d->mfiles;
			d->mbytes += st.st_size;
		}
		p->mode  = st.st_mode;
		p->mtime = st.st_mtime;
		p->atime = st.st_atime;
		p->ctime = st.st_ctime;
		p->size  = st.st_size;
		p->nlink = st.st_nlink;
		p->dev   = st.st_dev;
#if HAVE_ST_RDEV
		p->rdev  = st.st_rdev;
#else
		p->rdev  = 0;
#endif
		p->ino   = st.st_ino;
		p->uid   = st.st_uid;
		p->gid   = st.st_gid;
		strncpy (p->name, recp->d_name, sizeof(p->name)-1);

		if(p->mode==16804) p->mode=16877;  // dir or not dir ?

		p->name [sizeof(p->name)-1] = 0;
		if (uid == 0)
			p->execable = ((p->mode & 0111) != 0);
		else if (uid == p->uid)
			p->execable = p->mode >> 6 & 1;
		else if (mygroup (p->gid))
			p->execable = p->mode >> 3 & 1;
		else
			p->execable = p->mode & 1;
		p->tag = 0;
		p->info = 0;
		p->rinfo = 0;
		p->infodate = 0;
		p->infoflag = 0;
		p->dateflag = 0;
		++d->num;
	}

	d->topfile = d->curfile = 0;

	compdir = d;

	qsort ((char*)d->cat, (unsigned) d->num, sizeof (d->cat[0]), compfile);

	if (! dirname) 
	{
		retag (d, &olddir);
		free (olddir.cat);
		counttag (d);
	}
	closedir (f);
	return (1);
}

char *filesize2 (struct file *f)
{
	static char buf [9];

	switch (f->mode & S_IFMT) {
	case S_IFDIR:
		return ("<DIR>");
	case S_IFCHR:
		sprintf (buf, " %3d,%3d", f->rdev>>8&0377, f->rdev&0377);
		return (buf);
	case S_IFBLK:
		sprintf (buf, " %3d,%3d", f->rdev>>8&0377, f->rdev&0377);
		return (buf);
#ifdef S_IFIFO
	case S_IFIFO:
		return ("<FIFO>");
#endif
#ifdef S_IFNAM
	case S_IFNAM:
		return ("<NAME>");
#endif
#ifdef S_IFSOCK
	case S_IFSOCK:
		return ("<SOCKET>");
#endif
#ifdef S_IFLNK
	case S_IFLNK:
		return ("<LINK>");
#endif
	case S_IFREG:
		if(f->size>9999999)
		{
			double _size = f->size / 1000000;
			sprintf (buf, "%5.0f M", (double) _size);
		}
		else sprintf (buf, "%7ld", (long) f->size);
		return (buf);
	default:
		return ("???");
	}
}


char *filesize (struct file *f)
{
	static char buf [9];

	switch (f->mode & S_IFMT) {
	case S_IFDIR:
		return ("<DIR>");
	case S_IFCHR:
		sprintf (buf, " %3d,%3d", f->rdev>>8&0377, f->rdev&0377);
		return (buf);
	case S_IFBLK:
		sprintf (buf, " %3d,%3d", f->rdev>>8&0377, f->rdev&0377);
		return (buf);
#ifdef S_IFIFO
	case S_IFIFO:
		return ("<FIFO>");
#endif
#ifdef S_IFNAM
	case S_IFNAM:
		return ("<NAME>");
#endif
#ifdef S_IFSOCK
	case S_IFSOCK:
		return ("<SOCKET>");
#endif
#ifdef S_IFLNK
	case S_IFLNK:
		return ("<LINK>");
#endif
	case S_IFREG:
		sprintf (buf, "%12ld", (long) f->size);
		return (buf);
	default:
		return ("???");
	}
}


char *filemode (int m)
{
	static char buf [10];
	register char *p = buf;

	*p++ = m & S_IREAD ? 'r' : '-';
	*p++ = m & S_IWRITE ? 'w' : '-';
	*p++ = m & S_IEXEC ?
		(m & S_ISUID ? 's' : 'x') :
		(m & S_ISUID ? 'S' : '-');
	*p++ = m & S_IREAD >> 3 ? 'r' : '-';
	*p++ = m & S_IWRITE >> 3 ? 'w' : '-';
	*p++ = m & S_IEXEC >> 3 ?
		(m & S_ISGID ? 's' : 'x') :
		(m & S_ISGID ? 'S' : '-');
	*p++ = m & S_IREAD >> 6 ? 'r' : '-';
	*p++ = m & S_IWRITE >> 6 ? 'w' : '-';
	*p++ = m & S_IEXEC >> 6 ?
		(m & S_ISVTX ? 't' : 'x') :
		(m & S_ISVTX ? 'T' : '-');
	*p = 0;
	return (buf);
}

char *filedate (long d)
{
	static char buf [10];
	register char *p = buf;

	*p++ = d / 100000 + '0';
	*p++ = d / 10000 % 10 + '0';
	*p++ = '.';
	*p++ = d / 1000 % 10 + '0';
	*p++ = d / 100 % 10 + '0';
	*p++ = '.';
	*p++ = d / 10 % 10 + '0';
	*p++ = d % 10 + '0';
	return (buf);
}


int exist (char *name)
{
	struct stat st;

	if (stat (name, &st) < 0)
		return (0);
	switch (st.st_mode & S_IFMT) {
	case S_IFDIR:
		return ('d');
	case S_IFREG:
		return ('f');
	default:
		return ('?');
	}
}

long modtime (char *name)
{
	struct stat st;

	if (stat (name, &st) < 0)
		return (0);
	return (st.st_mtime);
}



void findfile (struct dir *d, char *name)
{
	/* set current file by name */
	register struct file *f;

	for (f=d->cat; f<d->cat+d->num; ++f)
		if (! strncmp (f->name, name, NAMESZ-1)) {
			d->curfile = f - d->cat;
			break;
		}
	while (d->topfile > d->curfile)
		d->topfile -= H;
	if (d->topfile < 0)
		d->topfile = 0;
	while (d->topfile + height() <= d->curfile)
		cur->topfile += H;
}



const char *extension (const char *p)
{
	const char *s;

	if (p[0]=='.' && p[1]=='.' && p[2]==0)
		return (p+2);
	s = 0;
	while (*++p)
		if (*p == '.')
			s = p;
	return (s ? s : p);
}

char *strtail (char *p, int delim, int maxlen)
{
	register char *s;

	/* return tail of string */
	for (s=p; *s; ++s);
	s -= maxlen;
	if (s <= p)
		return (p);
	for (; *s; ++s)
		if (*s == delim)
			return (s+1);
	return (s - maxlen);
}




void execute (void)
{
	register struct file *file = &cur->cat[cur->curfile];
	register char *name = file->name;
	int updir, dev = 0, ino = 0;

	switch (file->mode & S_IFMT) {
	case S_IFDIR:
		 //PrintString( stdscr, 0, 120,  "F1");
		if ((updir = !strcmp (name, ".."))) 
		{
			dev = cur->dev;
			ino = cur->ino;
		}
		if (chdir (name) < 0)
			break;
		setdir (cur, ".");
		break;
	}
}



int mystrcmp(char *s, char *t)
{
  int Result = 0;
  int s_length = 0;
  int t_length = 0;
  s_length = strlen(s);
  t_length = strlen(t);
  if(t_length <= s_length)
  {
    s += s_length - t_length;
    if(0 == strcmp(s, t))
    {
      Result = 1;
    }
  }

  return Result;
}

int ice_filetype (struct file *f, bool is_vfs )
{

	if(is_vfs)
	{
		char * s = f->name;
		s += strlen(f->name)-1;
		char end_char = *s;
		return end_char;
	}

	switch (f->mode & S_IFMT) {
	case S_IFDIR:
		if (! f->execable)
			return ('&');
		return ('/');
	case S_IFCHR:
		return ('$');
	case S_IFBLK:
		return ('#');
#ifdef S_IFIFO
	case S_IFIFO:
		return ('=');
#endif
#ifdef S_IFNAM
	case S_IFNAM:
		return ('@');
#endif
#ifdef S_IFSOCK
	case S_IFSOCK:
		return ('!');
#endif
#ifdef S_IFLNK
	case S_IFLNK:
		return ('~');
#endif
	case S_IFREG:
		if (f->execable)
			return ('*');
		return (' ');
	default:
		return ('?');
	}
}




