#include <sys/param.h>

#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include <curses.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <locale.h>
#include <sys/wait.h>
#include <sys/time.h>	
#include <stdlib.h>
#include <unistd.h>
#include <dirent.h>
#include <errno.h>
#include <fcntl.h>
#include <grp.h>
#include <pwd.h>
#include <setjmp.h>
#include <signal.h>
#include <string.h>
#include <wchar.h>

#define PATSZ          20       /* max length of pattern */

#define SORTNAME       1        /* sort by name */
#define SORTEXT        2        /* sort by extension */
#define SORTTIME       3        /* sort by time */
#define SORTSIZE       4        /* sort by size */
#define SORTSKIP       5        /* unsorted */

#define NAMESZ 256

struct file {
	char name [NAMESZ];     /* file name */
	char *info;             /* file info */
	char *rinfo;            /* file info in russian */
	unsigned execable:1;    /* file is executable for me */
	unsigned tag:1;         /* file tagged */
	unsigned link:1;        /* symbolic link */
	unsigned infoflag:1;    /* info is read */
	unsigned dateflag:1;    /* date is read from info file */
	unsigned long size;     /* file size */
	unsigned short mode;    /* access mode */
	unsigned short uid;     /* user ident */
	unsigned short gid;     /* group ident */
	unsigned short nlink;   /* number of file links */
	short dev;              /* device */
	short rdev;             /* special file parameters */
	unsigned long ino;      /* i-node */
	long mtime;    /* modification time */
	unsigned long atime;    /* access time */
	unsigned long ctime;    /* status change time */
	unsigned long infodate; /* date from info file */
};

struct file_tag {
	char name [NAMESZ];     /* file name */
};

struct dir {
	char cwd [MAXPATHLEN];  /* directory full name */
	char *shortcwd;         /* short printable directory name */
	short dev;              /* device */
	unsigned long ino;      /* i-node */
	struct file *cat;       /* catalog */
	int catsz;              /* length of catalog */
	int num;                /* number of files in cat */
	int topfile;            /* top file in the window */
	int curfile;            /* current file */
	int bottomfile;         /* bottom file */
 	char view;              /* view format */
	char status;            /* is it status window */
	char sort;              /* sort kind */
	char typesort;          /* sort by file types */
	char revsort;           /* reverse sorting */
	char alignext;          /* align extensions */
	int basecol;            /* base of window */
	int nfiles;             /* number of regular files */
	long nbytes;            /* number of bytes used */
	int mfiles;             /* number of matched files */
	long mbytes;            /* number of bytes matched */
	int tfiles;             /* number of tagged files */
	long tbytes;            /* number of bytes tagged */
	char pattern [PATSZ];   /* pattern for file names */
};


void ChangePanel(void);

struct dir left, right , vfs_left, vfs_right, *cur ;

typedef struct {
	char *name;
} DATA;

DATA temp , *TheArray;

int left_count_tag;
int right_count_tag;


/////////////////////// VFS

struct vfs_node
{
	long size;
	char name[256];
  time_t mtime;             

	struct vfs_node * next;
};

struct vfs_node_f5
{
	char source[512];
	char target[512];
	struct vfs_node_f5 * next;
};

struct vfs_path
{
	char name[512];
	struct vfs_path * next;
};

struct _ice_vfs
{
	struct vfs_node * List;
	struct vfs_node * List_Subdir;
	struct vfs_path * Path;
	struct vfs_node_f5 * List_F5;
	struct vfs_path * List_Dir_F5;

	char * Path_Dir;
	char * Path_Dir_Root;
	char * f5_root;

	int len;
	int len_subdir;
	int len_path;
	int del_count;

	bool endian_char;
	bool other_dir;
	bool bad_format;

	bool is_vfs;
	char file_name[256];
	char file_ext[256];


};

struct _ice_vfs ice_vfs_left,ice_vfs_right,*vfs_cur;

extern int compfile (const void *, const void *);
extern int comptype (const struct file *f);
extern int setdir (struct dir *, char *);
extern int ice_filetype (struct file *f, bool sadf);
extern void counttag (struct dir *);
extern int match2 (char *, char *);

extern void  vfs_setdir (struct dir *,char *,struct vfs_node * );
extern void ice_vfs_path_Pop(struct vfs_path ** );
extern void ice_vfs_path_Push(struct vfs_path **  , char * );
extern void vfs_get_subdir(char * ); 
extern void ice_vfs_init(char *);
extern char *filesize (struct file *);
extern char *filesize2 (struct file *);
extern char *timestr (long tim);
extern char * get_vfs_path_f5(struct vfs_path * , char * , int);
extern void vfs_get_subdir_f5(char *,char *,char *);
