/* mkdir -- make directories
   Copyright (C) 90, 1995-2001 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/* David MacKenzie <djm@ai.mit.edu>  */

#include "config.h"
#include <stdio.h>
#include <getopt.h>
#include <sys/types.h>
#include <syslog.h>

#include "system.h"
#include "dirname.h"
#include "error.h"
#include "makepath.h"
#include "modechange.h"
#include "quote.h"

/* The official name of this program (e.g., no `g' prefix).  */
#define PROGRAM_NAME "mkdir"

#define AUTHORS "David MacKenzie"

/* The name this program was run with. */
char *program_name;

/* If nonzero, ensure that all parents of the specified directory exist.  */
static int create_parents;

static struct option const longopts[] =
{
  {"mode", required_argument, NULL, 'm'},
  {"parents", no_argument, NULL, 'p'},
  {"verbose", no_argument, NULL, 'v'},
  {GETOPT_HELP_OPTION_DECL},
  {GETOPT_VERSION_OPTION_DECL},
  {NULL, 0, NULL, 0}
};

void
usage (int status)
{
}

int ice_mkdir (char * _new_dir)
{
  mode_t newmode;
  mode_t parent_mode;
  const char *specified_mode = NULL;
  const char *verbose_fmt_string = NULL;
  int errors = 0;
  int optc;

//  program_name = argv[0];
//  setlocale (LC_ALL, "");
//  bindtextdomain (PACKAGE, LOCALEDIR);
//  textdomain (PACKAGE);
//  atexit (close_stdout);

  create_parents = 1;


  newmode = S_IRWXUGO;
  {
    mode_t umask_value = umask (0);
    umask (umask_value);		/* Restore the old value. */
    parent_mode = (newmode & (~ umask_value)) | S_IWUSR | S_IXUSR;
  }

  if (specified_mode)
    {
      struct mode_change *change = mode_compile (specified_mode, 0);
      newmode &= ~ umask (0);
      if (change == MODE_INVALID)
{
	//error (1, 0, _("invalid mode %s"), quote (specified_mode));
}
      else if (change == MODE_MEMORY_EXHAUSTED)
	xalloc_die ();
      newmode = mode_adjust (newmode, change);
    }


      int fail = 0;

	    if (create_parents)
		{
		char *parents = dir_name (_new_dir);
		fail = make_path (parents, parent_mode, parent_mode, -1, -1, 1, verbose_fmt_string);
		free (parents);
		}

      if (fail == 0)
		{
		const char *dir = (const char *)_new_dir;
		int dir_created;
		fail = make_dir (dir, dir, newmode, &dir_created);
		if (fail)
			{
			/* make_dir already gave a diagnostic.  */
			}
		else if (!create_parents && !dir_created)
			{
			/* make_dir `succeeds' when DIR already exists.
			In that case, mkdir must fail, unless --parents (-p)
			was specified.  */
			//error (0, EEXIST, _("cannot create directory %s"),				quote (dir));
			fail = 1;
			}
		else if (verbose_fmt_string)
{
			//error (0, 0, verbose_fmt_string, quote (dir));
}	
		/* mkdir(2) is required to honor only the file permission bits.
			In particular, it needn't do anything about `special' bits,
			so if any were set in newmode, apply them with chmod.
			This extra step is necessary in some cases when the containing
			directory has a default ACL.  */
	
		/* Set the permissions only if this directory has just
			been created.  */
	
		if (fail == 0 && specified_mode && dir_created)
			{
			fail = chmod (dir, newmode);
			//if (fail)			error (0, errno, _("cannot set permissions of directory %s"),				quote (dir));
			}
		}

      if (fail)	errors = 1;

  return errors;
}
