#include <sys/types.h>	
#include <stdlib.h>			/* free() */
#include <string.h>			/* strcmp() */

#include "ice.h"


static HIST_ENTRY **history;/* command line history */
static int hs_alloc = 0;	/* number of allocated entries */
static int hs_cnt;			/* entries in use */
static int pn_index;		/* index for previous/next cmd */
						/* for temporary saving of the command line */

void hist_reset_index(void)
{
	pn_index = -1;
}

void hist_reconfig(void)
{
	int i;
	static HIST_ENTRY *storage;

	if (hs_alloc) {
		free(storage);
		free(history);
		free(panel_hist.hist);
	}

	hs_alloc = config_num(33);
	storage = (HIST_ENTRY *)emalloc(hs_alloc * sizeof(HIST_ENTRY));
	history = (HIST_ENTRY **)emalloc(hs_alloc * sizeof(HIST_ENTRY *));
	panel_hist.hist = (HIST_ENTRY **)emalloc(hs_alloc * sizeof(HIST_ENTRY *));
	for (i = 0; i < hs_alloc; i++) {
		history[i] = storage + i;
		US_INIT(history[i]->cmd);
	}

	hs_cnt = 0;
	hist_reset_index();
}





/*
 * hist_save() puts the command 'cmd' on the top
 * of the command history list.
 */
void hist_save(const char *cmd, int failed)
{
	int i;
	FLAG new;
	HIST_ENTRY *x, *top;

	hist_reset_index();

	new = 1;
	for (top = history[0], i = 0; i < hs_alloc; i++) {
		x = history[i];
		history[i] = top;
		top = x;
		if (i == hs_cnt) {
			hs_cnt++;
			break;
		}
		if (strcmp(USTR(top->cmd),cmd) == 0) {
			/* avoid duplicates */
			new = 0;
			break;
		}
	}
	if (new)
		us_copy(&top->cmd,cmd);
	top->failed = failed;
	
	history[0] = top;
}



