#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include <curses.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <locale.h>
#include <sys/wait.h>
#include <sys/time.h>	
#include <stdlib.h>
#include <unistd.h>
#include <dirent.h>
#include <errno.h>
#include <fcntl.h>
#include <grp.h>
#include <pwd.h>
#include <setjmp.h>
#include <signal.h>
#include <string.h>
#include <wchar.h>

#define STATFS(a, b, c, d )     statfs( a, b )
#define LONGLONG               long long
#define HAS_LONGLONG           1

#define MINIMUM( a, b ) ( ( (a) < (b) ) ? (a) : (b) )
#define MAXIMUM( a, b ) ( ( (a) > (b) ) ? (a) : (b) )

#define HAVE_RENAME

#ifdef TERMCAP
#define  KEY_BTAB         5000
#define  KEY_DOWN         5001
#define  KEY_UP           5002
#define  KEY_LEFT         5003
#define  KEY_RIGHT        5004
#define  KEY_END          5005
#define  KEY_HOME         5006
#define  KEY_NPAGE        5007
#define  KEY_PPAGE        5008
#define  KEY_DC           5009
#define  KEY_BACKSPACE    5010
#define  KEY_EIC          5011
#define  KEY_IC           5012
#define  KEY_DL           5013
#define  NO_HIGHLIGHT
#define  A_REVERSE        1 
#define  A_BLINK          2
#define  BELL             0x07
#undef   cbreak  
#define  cbreak()                    raw()
#define  beep()                      putchar( BELL )
#define  echochar( ch )              { addch( ch ); refresh(); }
#define  putp( str )                 tputs( str, 1, putchar )
#define  wnoutrefresh( win )         wrefresh( win )
#define  wgetch( win )               TermcapWgetch( win )
#define  vidattr( attr )             TermcapVidattr( attr )
#define  initscr()                   TermcapInitscr()
#define  endwin()                    TermcapEndwin()
#define  doupdate() 
#define  wattrset( win, attr ) 
#define  typeahead( file )
#define  keypad( win, flag )
#endif /* TERMCAP */

#ifndef KEY_BTAB
#define KEY_BTAB  0x1d
#endif

#ifndef KEY_END
#define KEY_END   KEY_EOL
#endif




#define DIR_COLOR        1
#define FILE_COLOR       2
#define STATS_COLOR      3
#define BORDERS_COLOR    4
#define MENU_COLOR       5
#define WINDIR_COLOR     6
#define WINFILE_COLOR    7
#define WINSTATS_COLOR   8
#define WINERR_COLOR     9
#define HIDIR_COLOR     10
#define HIFILE_COLOR    11
#define HIMENUS_COLOR   13
#define WINHST_COLOR    14
#define HST_COLOR       15
#define HIHST_COLOR     16
#define MTCH_COLOR      15
#define GLOBAL_COLOR    17
#define HIGLOBAL_COLOR  18

#define hist_initialize hist_reconfig

#define ERROR_MSG( msg )   Error( msg, __FILE__, __LINE__ ) 
#define WARNING( msg )     Warning( msg ) 
#define MESSAGE( msg )     Message( msg ) 
#define NOTICE( msg )      Notice( msg ) 
#define BLKSIZ             512  /* Blockgroesse fuer SVR3 */
#define BOOL       unsigned char
#define LF         10
#define ESC        27
#define PATH_LENGTH            1024

#define PRINT(ch) (iscntrl(ch) && (((unsigned char)(ch)) < ' ')) ? (ACS_BLOCK) : ((unsigned char)(ch)) 

#define ARRAY_SIZE(X)	(sizeof(X) / sizeof(X[0]))
#define TSET(X)			((X) ? 1 : ((X) = 1, 0))	/* test & set */
#define TCLR(X)			((X) ? ((X) = 0, 1) : 0)	/* test & clear */
#define CH_ESC			'\033'						/* ASCII escape */
#define CH_CTRL(X)		((X) & 0x1F)				/* ASCII ctrl-X */
#define IS_CHAR(X)		(((X) & 0xFF) == (X))		/* X is a char */
#define CMP(V1,V2) ((V1) == (V2) ? 0 : (V1) < (V2) ? -1 : 1)
#define STR(X)	STRINGIZE(X)
#define STRINGIZE(X)	#X

#define config_num(X) (*(const int *)pcfg[X])
#define config_str(X) ((const char *)pcfg[X])
#define config_layout \
	((const char *)pcfg[CFG_LAYOUT1 + config_num(CFG_LAYOUT)])


typedef unsigned short int FLAG;	
typedef short int CODE;				

#define SDSTRING_LEN 15
typedef struct {
	char *SDname;					
	char SDmem[SDSTRING_LEN + 1];	
} SDSTRING;

typedef struct {
	char *USstr;		
	size_t USalloc;		
} USTRING;

typedef struct {
	FLAG curses;		
	FLAG wait;			
	int scrcols;		
	int pancols;		
	int scrlines;		
	int panlines;		
	int textline_area;	
	int date_len;		
} DISPLAY;

typedef struct {
	const char *login_at_host;	
	char *shell;				
	const char *homedir;		
	pid_t pid;					
	mode_t umask;				
	FLAG isroot;				
	FLAG admin;					
	CODE shelltype;				
} TERM_DATA;


typedef struct {
	const char *prompt;	
	int promptlen;		
	USTRING line;		
	int size;			
	int curs;			
	int offset;			


	struct {
		USTRING save_line;
		int save_size;
		int save_curs;
		int save_offset;
	} undo [10];	
	int undo_base;	
	int undo_levels;
	int redo_levels;
} TEXTLINE;

typedef struct {
	char line[25];	
	int size;		
	int curs;		
	FLAG changed;	
} INPUTLINE;


typedef struct {
	const char *text;	
	const char *info;
	void (*fn)(void);
} EXTRA_LINE;

typedef struct {
	int cnt, top;	
	int curs, min;	
	CODE type;		
	FLAG norev;		
	EXTRA_LINE *extra;	
	INPUTLINE *filter;	
	CODE filtering;		
} PANEL_DESC;

#define FT_NA				12





#define CFG_VARIABLES		34
#define CFG_FRAME			 0
#define CFG_LAYOUT1          5
#define CFG_XTERM_TITLE		 2
#define CFG_PROMPT			 4
#define CFG_LAYOUT			 8
#define CFG_SHELLPROG		14


#define US_INIT(X)	do { (X).USstr = 0; (X).USalloc = 0; } while (0)
#define PUSTR(X)	((X)->USstr)
#define  USTR(X)	((X).USstr)
#define SD_INIT(X)	do { (X).SDname = 0; (X).SDmem[0] = '\0';} while (0)
#define PSDSTR(X)	((X)->SDname ? (X)->SDname : (X)->SDmem)
#define  SDSTR(X)	((X).SDname  ? (X).SDname  : (X).SDmem)





typedef struct {
	const char *var;	
	const char *help;	
	void *table;		
	unsigned int isnum:1;
	unsigned int changed:1;
	unsigned int saveit:1;	
} CONFIG_ENTRY;

typedef struct {
	USTRING cmd;			
	FLAG failed;			
} HIST_ENTRY;

typedef struct {
	PANEL_DESC *pd;
	HIST_ENTRY **hist;		
} PANEL_HIST;



extern const void *pcfg[CFG_VARIABLES];

extern DISPLAY display;
extern TERM_DATA term_data;
extern TEXTLINE *textline;	
extern TEXTLINE line_cmd, line_dir, line_tmp;
extern PANEL_DESC *panel;	

 



#define mcheck(array, type, bound, quant, index)\
	if ((index) >= (bound)) (array) = (type) realloc (array,\
		(int) ((bound) += (quant)) * (int) sizeof (*(array)))

const char *extension (const char *p);
char *strtail (char *p, int delim, int maxlen);

int CountDisplay;
char  * fn;

bool EDIT ;

struct _ice_panel
{
	bool GoUp;
	bool GoDown;
	char DirUp[512];
	char DirDown[512];
	char CurFileDown[512];

	int  list_type;
	int  sort_type;
	int  pos_menu;
	bool reverse;
};

struct _ice_panel panel_left , panel_right , *cur_panel;

struct _console
{
	int  f7_line[256];
	int  f7_line2[256];
	char f7_buf [512];
	int  f7_pos  ;
	int  f7_start ;
	int  f7_length ;

	int  main_line[512];
	int  main_line2[512];
	char main_buf [1024];
	int pos  ;
	int start ;
	int length ;

	char utf_symbols1[66];
	char utf_symbols2[66];
	int  key_symbols1[66];
	int  key_symbols2[66];

};

struct _console ice_console;

struct _color_scheme
{
	int type;
	int background_editor_color;
	char ice_color[10];
	int tag_color;
	int file_color;
	int file2_color;
	int back_color;
};

struct _color_scheme color_scheme;

extern unsigned int window_changed_size;

enum _COLORS
{
	BLUE_COLOR=1,
	BLACK_COLOR,
	WHITE_COLOR,
	GREEN_COLOR,
	YELLOW_COLOR,
	CYAN_COLOR,
	RED_COLOR
};

extern PANEL_HIST panel_hist;
extern void StartColors(void);

extern WINDOW *file_window;
extern int       mode;
extern char      message[];
extern char *getenv(const char *);
extern void Display(void);
extern BOOL Match(char *file_name);
extern void Error(char *msg, char *module, int line);
extern void Warning(char *msg);
extern void Notice(char *msg);
extern int  SystemCall(char *command_line);
extern void Quit(void);
extern void Message(char *msg);
extern char *CTime(time_t f_time, char *buffer);
extern void Print(WINDOW *, int, int, char *, int);
extern time_t Mktime(struct tm *tm);
extern int  TermcapWgetch(WINDOW *win);
extern void TermcapVidattr(int attr );
extern void TermcapInitscr(void);
extern void TermcapEndwin(void);

extern int  WAddStr(WINDOW *win, char *str);
extern int  AddStr(char *str);
extern char *Strdup(const char *s);
extern void StrCp(char *dest, const char *src);
extern int  Getch(void);
extern char *Getcwd(char *buffer, unsigned int len);


extern char *estrdup(const char *);
extern void pathname_set_directory(const char *);
extern char *pathname_join(const char *);
extern size_t dequote_txt(const char *,  size_t, char *);
extern ssize_t read_fd(int, char *, size_t);

extern bool IsDir(void);

// exec.c
extern char * base_name (char const *);
extern void us_copy(USTRING *, const char *);
extern void exec_shell_reconfig(void);
extern void exec_prompt_reconfig(void);
extern void us_cat(USTRING *pustr, ...);
extern void xterm_title_set(int , const char *);
extern void tty_reset(void);
extern void tty_press_enter(void);
extern void tty_setraw(void);
extern int  get_cwd_us(USTRING *);
extern void us_setsize(USTRING *, size_t );
extern void us_copyn(USTRING *, const char *, size_t );
extern const char *dir_tilde(const char *);
extern void curses_stop(void);
extern void hist_save(const char *, int );
extern void curses_restart(void);

// history.c
extern void * emalloc(size_t );
extern void win_remark(const char *);

// out.c
extern void signal_initialize(void);

// main.c
extern void edit_term_init (void);
extern void SetColors(int,int);
extern int ice_key_code (void);
extern int GetUtf(int,int);
extern void userdata_initialize(void);
extern void config_initialize(void);
extern void exec_initialize(void);
extern void hist_initialize(void);
extern void xterm_title_initialize(void);
extern void tty_initialize(void);
extern void curses_initialize(void);
extern void UpdateColor(void);
extern int demolib_init_terminal (int , int );
extern int height(void);
extern void InitBottomFiles(void);
extern void ice_manage(void);

// user.c
extern void sd_reset(SDSTRING *);
extern void sd_copy(SDSTRING *, const char *);

// string.c

extern void * erealloc(void *, size_t );

// util
extern void xterm_title_restore(void);
extern void tty_pgrp_reset(void);
extern int execute_cmd(const char *);
extern void win_edit(void);
extern void us_resize(USTRING *, size_t );

// dialog.c
extern void draw_dialog_f1(char *) ;
extern void draw_dialog_f2(char * ) ;
extern void draw_dialog_f7(int ,char * ) ;
extern void draw_dialog_warning(char * ) ;
extern void draw_dialog(bool , char * );
extern void draw_dialog_f5(bool , char * , char * ) ;
extern void draw_dialog_f9(char * );
extern int mystrcmp(char *, char *);
extern void draw_command_line(int); 

// keyboard.c
extern int ice_cp (char * , char *);
extern void ice_rm (char * );
extern bool dialog_action_f7(char * ) ;
extern void ICE_KEY_ENTER(void);
extern void ICE_KEY_ENTER_VFS(void);
extern void ICE_KEY_LEFT(void);
extern void ICE_KEY_LEFT_VFS(void);
extern void KEY_RIGHT_(void) ;
extern int  ice_unzip(int,char * my_argv[]);
extern void files_enter(char * );
extern void ice_init_console(void);
extern bool dialog_action(char *  , int , bool  , char *  , char * ); 
extern int editfile(char *,bool ) ;
extern int ice_create_file (char * );
extern void KEY_F5(int ,char * );
extern void KEY_F6(int ,char * );
extern int ice_mkdir (char * );
extern void draw_dialog_waiting(char * ,char * );
extern void close_history(void); 
extern void unset_interactive_mode(void);
extern void DisplayPanels(void);
extern int dialog_comand_line(int,bool ,bool ); 
extern void GoNewDir(void);
extern int GetCurrentTag(void);
extern void ice_mv(char * , char * );

// screen.c
extern void GetCommandBuffer(void);
extern void GetMainCommandBuffer(void);

// slang.c
extern int ttysize(void);
extern void clear_entire_screen(void);
extern void reset_window(void);
