#include <sys/types.h>	
#include <sys/stat.h>	
#include <stdio.h>		
#include <stdlib.h>		
#include <string.h>		
#include <unistd.h>		
#include <locale.h>
#include <regex.h>
#include <signal.h>
#include <limits.h>
#include <syslog.h>

#include "ice.h"
#include "dir.h"
#include "edit/ne.h"

#define LOCALE_REGEX "\\(UTF-?8\\)\\|\\(ISO-?8859-?\\)\\(1?[0-9]\\)"

int editfile(char *filename,bool editable) 
{

	int  c;
	bool _utf 	   = FALSE;
	int _insert_count = 0;
	int  _utf_code = 0;


  edit_term_init();
	set_terminal_window(ne_lines-1);

	if (!new_buffer()) exit(1);
	clear_buffer(cur_buffer);

	re_set_syntax(
		RE_CONTEXT_INDEP_ANCHORS |
		RE_CONTEXT_INDEP_OPS	| RE_HAT_LISTS_NOT_NEWLINE |
		RE_NEWLINE_ALT			| RE_NO_BK_PARENS				|
		RE_NO_BK_VBAR			| RE_NO_EMPTY_RANGES
	);

//	SetColors(BLUE_COLOR,WHITE_COLOR);
//	SetColors(BLACK_COLOR,WHITE_COLOR);
	SetColors(color_scheme.background_editor_color,WHITE_COLOR);

//  SLsmg_set_color (0);
	clear_entire_screen();
	do_action(cur_buffer, OPEN_A, -1, (unsigned char *)filename);
	refresh_window(cur_buffer);

	EDIT=TRUE;
	cursor_on();	
	cur_buffer->opt.read_only=editable;

	while(EDIT) {

		refresh_window(cur_buffer);
		draw_status_bar();
		move_cursor(cur_buffer->cur_y, cur_buffer->cur_x);
		fflush(stdout);

		c = ice_key_code ();


		if (_utf)
		{
			_utf = FALSE;
			c = GetUtf(c,_utf_code);

		}

		if((c==208)||(c==209)) // utf
		{
			_utf = TRUE;
			_utf_code = c ;
			continue;
		}
		else if ((c>=513 && c < 525)|| c>65000) {} 	// F1 - F12
		else if(c==13) // Enter
			print_error(do_action(cur_buffer, INSERTLINE_A, -1, NULL));
		else if((c==27) ||(c>250 && c<300))  // navigation 
			print_error(do_action(cur_buffer, c, -1, NULL));
		else 			// print
		{
			//_insert_action=TRUE;
			if(_insert_count < 4000)
				print_error(do_action(cur_buffer, INSERTCHAR_A, c, NULL));
			_insert_count ++ ;
		}

	}

	return 0;
}

list buffers = { (node *)&buffers.tail, NULL, (node *)&buffers.head };
list clips = { (node *)&clips.tail, NULL, (node *)&clips.head };
list macros = { (node *)&macros.tail, NULL, (node *)&macros.head };

buffer *cur_buffer;
int turbo;

buffer *new_buffer(void) {
	buffer *b = alloc_buffer(cur_buffer);

	if (b) 
	{
		clear_buffer(b);
		if (cur_buffer) add(&b->b_node, &cur_buffer->b_node);
		else add_head(&buffers, &b->b_node);
		cur_buffer = b;
	}

	return b;
}

int delete_buffer(void) {

	buffer *nb = (buffer *)cur_buffer->b_node.next, *pb = (buffer *)cur_buffer->b_node.prev;

	rem(&cur_buffer->b_node);
	free_buffer(cur_buffer);

	if (pb->b_node.prev) {
		cur_buffer = pb;
      return TRUE;
	}

	if (nb->b_node.next) {
		cur_buffer = nb;
		return TRUE;
	}

	return FALSE;
}

int Init(char *configuration_file)
{
  cbreak();  
  keypad( stdscr, FALSE );  
  clearok(stdscr, TRUE);  
  leaveok(stdscr,FALSE);

  return( 0 );
}


int GetUtf(int c,int _utf_code)
{
	if (_utf_code==208)
	{
		return (896+c);
	}
	if (_utf_code==209)
	{
		return (960+c);
	}
	return (960+c);
}

void ice_init_console(void)
{
	char utf_symbols1[66] = {-48,-48,-48,-48,-48,-48,-47,-48,-48,-48,-48,-48,-48,-48,-48,-48,-48,-47,-47,-47,-47,-47,-47,-47,-47,-47,-47,-47,-47,-47,-47,-47,-47,-48,-48,-48,-48,-48,-48,-48,-48,-48,-48,-48,-48,-48,-48,-48,-48,-48,-48,-48,-48,-48,-48,-48,-48,-48,-48,-48,-48,-48,-48,-48,-48,-48};
	char utf_symbols2[66] = {-80,-79,-78,-77,-76,-75,-111,-74,-73,-72,-71,-70,-69,-68,-67,-66,-65,-128,-127,-126,-125,-124,-123,-122,-121,-120,-119,-118,-117,-116,-115,-114,-113,-112,-111,-110,-109,-108,-107,-127,-106,-105,-104,-103,-102,-101,-100,-99,-98,-97,-96,-95,-94,-93,-92,-91,-90,-89,-88,-87,-86,-85,-84,-83,-82,-81};
	int  key_symbols1[66] = {208,208,208,208,208,208,209,208,208,208,208,208,208,208,208,208,208,209,209,209,209,209,209,209,209,209,209,209,209,209,209,209,209,208,208,208,208,208,208,208,208,208,208,208,208,208,208,208,208,208,208,208,208,208,208,208,208,208,208,208,208,208,208,208,208,208};
	int  key_symbols2[66] = {176,177,178,179,180,181,145,182,183,184,185,186,187,188,189,190,191,128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143,144,145,146,147,148,149,129,150,151,152,153,154,155,156,157,158,159,160,161,162,163,164,165,166,167,168,169,170,171,172,173,174,175};

	int i;
	for(i=0;i<66;i++)
	{
		ice_console.utf_symbols1[i] = utf_symbols1[i];
		ice_console.utf_symbols2[i] = utf_symbols2[i];
		ice_console.key_symbols1[i] = key_symbols1[i];
		ice_console.key_symbols2[i] = key_symbols2[i];
	}

	for(i=0;i<512;i++)
	{
		ice_console.main_line[i]=0;
		ice_console.main_line2[i]=0;
	}				

	for(i=0;i<1024;i++) ice_console.main_buf[i]=0;

	ice_console.start=ice_console.pos=ice_console.length=0;

	panel_left.list_type=panel_left.sort_type=panel_left.pos_menu=panel_right.sort_type=panel_right.pos_menu=0;
	panel_left.sort_type=panel_right.sort_type=panel_right.list_type=1;
	panel_left.reverse=panel_right.reverse=ice_vfs_left.is_vfs=ice_vfs_right.is_vfs=FALSE;

	vfs_cur = &ice_vfs_left;
	
}



int main(int argc, char *argv[])
{
	int i;
	char *skiplist, *locale;


/*
	int my_argc=4;
	char *my_argv[4]={"libtar","-z","-t","/000/0.tar"};
	int result = ice_tar(my_argc,  my_argv);
	exit(0);
*/

	clip_desc *cd;

	term_data.pid = getpid();
	term_data.umask = umask(0777);
	umask(term_data.umask);
	term_data.isroot = geteuid() == 0;  /* 0 or 1 */;

	signal_initialize();	
	userdata_initialize();	
	config_initialize();	
	exec_initialize();
	hist_initialize();
	xterm_title_initialize();
	tty_initialize();
	curses_initialize();

	color_scheme.type=1;
	UpdateColor();

	if ( !(skiplist = calloc( argc, 1 )))  /* We need this many flags. */
	  exit(1);                             /* This would be bad.       */

    if (-1 == demolib_init_terminal (1, 1))     return 1;

	locale = setlocale(LC_ALL, "");
	for(i = 0; i < 256; i++) localised_up_case[i] = toupper(i);

	if (locale) {
		struct re_pattern_buffer re_pb;
		struct re_registers re_reg;
		memset(&re_pb, 0, sizeof re_pb);
		memset(&re_reg, 0, sizeof re_reg);

		re_pb.translate = (unsigned char *)localised_up_case;
		re_compile_pattern(LOCALE_REGEX, strlen(LOCALE_REGEX), &re_pb);
		if (re_search(&re_pb, locale, strlen(locale), 0, strlen(locale), &re_reg) >= 0) {
			if (re_reg.start[1] >= 0) io_utf8 = TRUE;
		}
		free(re_reg.start);
		free(re_reg.end);
	}

	if (!new_buffer()) exit(1);
	clear_buffer(cur_buffer);
	if (!(cd = alloc_clip_desc(INT_MAX, 0))) exit(1);

	signal(SIGWINCH, handle_winch);

	ice_init_console();

	left.curfile=0;
  left.sort = SORTNAME;
  left.typesort = 1;
  left.alignext = 1;
	panel_left.sort_type=0;

	right.curfile=0;
	right.sort = SORTEXT;
	right.typesort = 1;
	right.alignext = 1;
	right.bottomfile=2*height();
	panel_right.sort_type=1;

 	setdir (&right, "/");
 	setdir (&left, "/");
	cur = &left;
	cur_panel = &panel_left; 
	InitBottomFiles();

	ice_manage();

	return 0;
}
