#define HAVE_TERM_H 1

#include <string.h>	
#include <sys/types.h>	
#include <ctype.h>		/* iscntrl() */
#include <stdarg.h>		/* va_list */
#include <string.h>		/* strcpy() */
#ifdef HAVE_NCURSES_H
# include <ncurses.h>	/* initscr() */
#else
# include <curses.h>
#endif
#ifdef HAVE_TERM_H
# include <term.h>		/* enter_bold_mode */
#else
# ifdef HAVE_NCURSES_TERM_H
#  include <ncurses/term.h>
# endif
#endif

#include "ice.h"


#ifndef A_NORMAL
# define A_NORMAL 0
#endif

#ifndef A_BOLD
# define A_BOLD A_STANDOUT
#endif

#ifndef A_REVERSE
# define A_REVERSE A_STANDOUT
#endif

#ifndef A_UNDERLINE
# define A_UNDERLINE A_STANDOUT
#endif

#ifndef ACS_HLINE
# define ACS_HLINE '-'
#endif

/* safe addch() */
#define ADDCH(X)	addch(iscntrl(X) ? '?' | attrb : (X))

#define BLANK(X)	do { char_line(' ',X); } while (0)

static const char
	*layout_panel,		/* layout: file panel part */
	*layout_line;		/* layout: info line part */
static const char
	*info_remark = 0,	/* this remark is in the information line */
	*info_warnmsg = 0;	/* this warning is in the information line */

/* pos_xxx are used for win_position() control */
static CODE pos_resize = 0;	/* --( COLSxLINES )-- window size */
static int prevkey = 0;			/* previously pressed key */
static chtype attrr, attrb;		/* reverse and bold or substitutes */
static int framechar;			/* panel frame character
								  (note that ACS_HLINE is an int) */

static void win_info(void);		/* defined below */


void win_frame_reconfig(void)
{
	switch(config_num(0)) {
	case 0:
		framechar = '-';
		break;
	case 1:
		framechar = '=';
		break;
	default:
		framechar = ACS_HLINE;
	}
}

void curses_stop(void)
{
	clear();
	refresh();
	endwin();
	display.curses = 0;
}

void txt_printf(const char *format, ...)
{
	va_list argptr;

	display.wait = 1;
	if (display.curses) {
		curses_stop();
		puts("\nReconfiguring ");
	}

	va_start(argptr,format);
	vprintf(format,argptr);
	va_end(argptr);
	fflush(stdout);
}



void win_layout_reconfig(void)
{
	static USTRING layout = { 0,0 };
	FLAG fld;
	char ch, *pch;

	us_copy(&layout,config_layout);

	/* split layout to panel fields and line fields */
	layout_panel = USTR(layout);
	for (fld = 0, pch = USTR(layout); (ch = *pch); pch++)
		if (!TCLR(fld)) {
			if (ch == '$')
				fld = 1;
			else if (ch == '|') {
				*pch = '\0';
				layout_line = pch + 1;
				return;	/* success */
			}
		}

	layout_line = "$m $p $o";
	txt_printf("CONFIG: Incorrect LAYOUT syntax: missing bar '|'\n");
}

void curses_initialize(void)
{
	if (TCLR(display.wait))
		tty_press_enter();

	initscr();			/* restores signal dispositions on FreeBSD ! */
	signal_initialize();	/* FreeBSD initscr() bug workaround ! */
	raw();
	nonl();
	noecho();
	keypad(stdscr,TRUE);
#ifdef HAVE_NOTIMEOUT
	notimeout(stdscr,TRUE);
#endif
	clear();
	refresh();
	display.curses = 1;

	if (enter_reverse_mode && *enter_reverse_mode)
		attrr = A_REVERSE;
	else
		attrr = A_STANDOUT;
	if (enter_bold_mode && *enter_bold_mode)
		attrb = A_BOLD;
	else if (enter_underline_mode && *enter_underline_mode)
		attrb = A_UNDERLINE;
	else
		attrb = A_STANDOUT;

	win_frame_reconfig();
	win_layout_reconfig();
}

void curses_restart(void)
{
	if (TCLR(display.wait))
		tty_press_enter();

	reset_prog_mode();
	touchwin(stdscr);
	display.curses = 1;
}


/****** keyboard input functions ******/

/*
 * get next input char, no processing except screen resize event,
 * use this input function to get unfiltered input
 */
int kbd_getraw(void)
{
	int key, retries;

	for (;/* until return */;) {
		retries = 10;
		do {
			if (--retries < 0){
					// err_exit("Cannot read the keyboard input");
				}
			key = getch();
		} while (key == ERR);
#ifdef KEY_RESIZE
		if (key == KEY_RESIZE) {
			pos_resize = 2;
			continue;
		}
#endif
#ifdef KEY_SUSPEND
		if (key == KEY_SUSPEND)
			return CH_CTRL('Z');
#endif
		if (key == KEY_BACKSPACE)
			return CH_CTRL('H');
		if (key == KEY_ENTER)
			return CH_CTRL('M');
		if (key == CH_CTRL('G'))
			return CH_CTRL('C');
		if (key == KEY_DC)
			return '\177';
		if (key == KEY_IL)
			return KEY_IC;
		return key;
	}
}

/*
 * get next input char, not saved for kbd_esc() usage,
 * use this function to get a single key unrelated to the rest
 * of input stream (e.g. "press any key to continue")
 */
static int
kbd_getany(void)
{
	int key;

	while ((key = kbd_getraw()) == CH_CTRL('L'))
		/* redraw screen */
		wrefresh(curscr);
	return key;
}


void win_remark_fmt(const char *format, ...)
{
	va_list argptr;
	static char buff[160];

	if (!display.curses) {
		txt_printf("BUG: win_remark_fmt(\"%s\",...)\n"
		  "     called in text mode\n",format);
		return;
	}

	va_start(argptr,format);
	vsnprintf(buff,sizeof(buff),format,argptr);
	va_end(argptr);
	info_remark = buff;
	win_info();
}

/* get next input char */
int kbd_input(void)
{
	int ch;
	static int key = 0;

	/* show ASCII code */
	if (info_remark == 0 && textline != 0) {
		ch = (unsigned char)USTR(textline->line)[textline->curs];
		if (ch != '\0' && iscntrl(ch)) {
			if (ch >= 1 && ch <= 26)
				win_remark_fmt("Ctrl-%c  ASCII %d",ch + 'A' - 1,ch);
			else
				win_remark_fmt("ASCII %d",ch);
		}
	}

	prevkey = key;
	key = kbd_getany();	
	/* <esc> 1 --> <F1>, <esc> 2 --> <F2>, ... <esc> 0 --> <F10> */
	if (prevkey == CH_ESC && IS_CHAR(key) && isdigit(key)) {
		prevkey = 0;
		key = KEY_F((key - '0' + 9) % 10 + 1);
	}

	/* dismiss remark (if any) */
	if (info_remark) {
		info_remark = 0;
		win_info();
	}

	return key;
}

/* was the previous key an ESC ? */
int
kbd_esc(void)
{
	return prevkey == CH_ESC;
}

/****** output functions ******/




/* write a line of repeating chars */
static void
char_line(int ch, int cnt)
{
	while (cnt-- > 0)
		addch(ch);
}

void win_frame(void)
{
	move(1,0);
	char_line(framechar,display.scrcols);
	move(display.panlines + 2,0);
	char_line(framechar,display.scrcols);
}


#define OPT_NOPAD	1	/* do not pad */
#define OPT_NOCONT	2	/* do not write continuation mark '>' */
#define OPT_SQUEEZE	4	/* squeeze long str in the middle: abc...xyz */
static int putstr_trunc(const char *str, int maxwidth, int options)
{
	int i, ch, len, dots, part1, part2;

	if (maxwidth <= 0)
		return 0;

	if (options & OPT_SQUEEZE) {
		len = strlen(str);
		if (len > maxwidth) {
			dots = maxwidth >= 6 ? 4 : 1;
			part2 = 5 * (maxwidth - dots) / 8;
			part1 = maxwidth - dots - part2;
			putstr_trunc(str,part1,OPT_NOCONT);
			char_line('.',dots);
			putstr_trunc(str + len - part2,part2,0);
			return maxwidth;
		}
		/* else: SQUEEZE option is superfluous --> ignored */
	}

	for (i = 0; i < maxwidth - 1; i++)
		if ( (ch = (unsigned char)str[i]) )
			ADDCH(ch);
		else {
			/* EOL */
			if (options & OPT_NOPAD)
				return i;
			BLANK(maxwidth - i);
			return maxwidth;
		}

	ch = (unsigned char)str[i];
	if (ch == '\0') {
		if (options & OPT_NOPAD)
			return i;
		addch(' ');
	}
	else if (str[i + 1] && !(options & OPT_NOCONT))
		addch('>' | attrb);	/* continuation mark in bold font */
	else
		ADDCH(ch);
	return maxwidth;
}



/* information line */
static void win_info(void)
{
}

/* warning message - can be used also in text mode */
void win_warning(const char *msg)
{
	if (!display.curses) {
		putchar('\n');
		fflush(stdout);
		puts(msg);
		return;
	}

	/*
	 * win_info() is responsible for this screen area,
	 * real work is performed there
	 */
	info_warnmsg = msg;
	win_info();
	kbd_getany();
	info_warnmsg = 0;
	win_info();
}




void win_remark(const char *str)
{
	if (!display.curses) {
		txt_printf("BUG: win_remark(\"%s\")\n"
		  "     called in text mode\n",str);
		return;
	}

	info_remark = str;
	win_info();
}


void win_edit(void)
{
	int width, offset;

	move(display.panlines + 5,0);

	/* special case: no textline */
	if (textline == 0) {
		clrtobot();
		return;
	}

	/* special case: empty line (prompt not highlighted) */
	if (textline->size == 0) {
		addstr(textline->prompt);
		clrtobot();
		return;
	}

	/* prompt (or continuation mark) */
	offset = textline->offset;
	width = display.textline_area;
	if (offset == 0) {
		addstr(textline->prompt);
		width -= textline->promptlen;
	}
	else {
		attrset(attrb);
		addch('<');
		width--;
	}
	attrset(A_NORMAL);
	putstr_trunc(USTR(textline->line) + offset,width + 1,0);
}






