#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>

#include "slang/slang.h"
#include "ice.h"
#include "dir.h"

#define TIMEOUT 2		       /* 2/10 of a second */

#define MAX_BUF_LEN	256

typedef struct
{
   char buf[MAX_BUF_LEN];
   char *prompt;
}
Our_Client_Data_Type;

SLang_Load_Type *SLns_allocate_load_type (char *name, char *namespace_name)
{
   SLang_Load_Type *x;

   if (NULL == (x = (SLang_Load_Type *)SLmalloc (sizeof (SLang_Load_Type))))
     return NULL;
   memset ((char *) x, 0, sizeof (SLang_Load_Type));

   if (name == NULL) name = "";

   if (NULL == (x->name = SLang_create_slstring (name)))
     {
        SLfree ((char *) x);
        return NULL;
     }

   if (namespace_name != NULL)
     {
        if (NULL == (x->namespace_name = SLang_create_slstring (namespace_name)))
          {
             SLang_free_slstring (x->name);
             SLfree ((char *) x);
             return NULL;
          }
     }

   return x;
}

SLang_Load_Type *SLallocate_load_type (char *name)
{
   return SLns_allocate_load_type (name, NULL);
}

static int init_readline (void)
{
   fflush (stdout);
   return 0;
}


static int readline (char *prompt, char *buf)
{
   fputs (prompt, stdout);  
   fflush (stdout);
   if (NULL == fgets (buf, MAX_BUF_LEN, stdin)) return -1;
   return (int) strlen (buf);
}

int read_input (void)
{
   int r = SLtt_Screen_Rows  ;
   SLsmg_gotorc (r-5, 0);

//   SLang_init_slfile ();

   init_readline ();


   SLang_Load_Type *x;
   Our_Client_Data_Type client_data;

   if (NULL == (x = SLallocate_load_type ("<readline>")))     return -1;
   client_data.prompt = "Ice> ";

   x->client_data = (VOID_STAR) &client_data;

//   x->read = read_using_readline; /* function to call to perform the read */

//   Our_Client_Data_Type *client_data2;


   Our_Client_Data_Type *client_data2;

   client_data2 = (Our_Client_Data_Type *) x->client_data;
   if (-1 == readline (client_data2->prompt, client_data2->buf)) return 0;


//   SLang_load_object (x);
   return 0;
}


static void demolib_exit (int sig)
{
   SLang_reset_tty ();
   SLsmg_reset_smg ();

   if (sig)
     {
	fprintf (stderr, "Exiting on signal %d\n", sig);
	exit (1);
     }
   exit (sig);
}

#ifdef SIGTSTP
static void sigtstp_handler (int sig)
{
   demolib_exit (sig);
}
#endif

#ifdef SIGINT
static void sigint_handler (int sig)
{
   demolib_exit (sig);
}
#endif

static void init_signals (void)
{
#ifdef SIGTSTP
   SLsignal (SIGTSTP, sigtstp_handler);
#endif
#ifdef SIGINT
   SLsignal (SIGINT, sigint_handler);
#endif
}

static void exit_error_hook (char *fmt, va_list ap)
{
   SLang_reset_tty ();
   SLsmg_reset_smg ();
   
   vfprintf (stderr, fmt, ap);
   fputc ('\n', stderr);
   exit (1);
}

   


int demolib_init_terminal (int tty, int smg)
{
   SLang_Exit_Error_Hook = exit_error_hook;
   
   /* It is wise to block the occurance of display  related signals while we are 
    * initializing. 
    */
   SLsig_block_signals ();
   
   SLtt_get_terminfo ();
   
   /* SLkp_init assumes that SLtt_get_terminfo has been called. */
   if (tty && (-1 == SLkp_init ()))
     {
	SLsig_unblock_signals ();
	return -1;
     }
   
   init_signals ();
   
   if (tty) SLang_init_tty (-1, 0, 1);
#ifdef REAL_UNIX_SYSTEM
   if (tty) SLtty_set_suspend_state (1);
#endif
   if (smg)
     {
	(void) SLutf8_enable (-1);

	if (-1 == SLsmg_init_smg ())
	  {
	     SLsig_unblock_signals ();
	     return -1;
	  }
     }

	
   SLsig_unblock_signals ();

   return 0;
}








static void init_colors (void);



static int slang_getch (void)
{
   int ch;
   
   while (0 == SLang_input_pending (1000))
     continue;
   
   ch = SLang_getkey ();
 
   if (ch == 033)		       // escape 
   {
		if (0 == SLang_input_pending (TIMEOUT))
	  	return 033;
   }
   
   SLang_ungetkey (ch);
   
   return SLkp_getkey ();
}


int ice_key_code ()
{

	_shift_key_sum.sum=0;
	int key = slang_getch ();
	if(key>65000)
	{
		if(_shift_key_sum.sum==7266)  // shift + F7 
		{
				return -444;	
		}
		else if(_shift_key_sum.sum==6227) // shift + F5
		{
				return -445;	
		}
		else if(_shift_key_sum.sum==6746) // shift + F6
		{
				return -446;	
		}
		return -500;
	}
    else return key;

}

void slang_test (void)
{   
   if (-1 == demolib_init_terminal (1, 1))     return ;

   init_colors ();

//   my_box_test ();

//   int ch = SLkp_getkey ();


	
//	 Bold_Vid_Str = SLtt_tgetstr("md");

   (void) SLtt_set_mouse_mode (1, 0);

//   if (argc <= 1)
//     menu_loop ();
/*   
   do
     {
	argc--;
	argv++;
	     
	if (-1 == select_menu_item (atoi (*argv)))
	  menu_loop ();
     }
   while (argc > 1);
*/	     
   demolib_exit (0);
   return ;
}

static void color_test (void);

typedef struct
{
   char *name;
   void (*funct) (void);
}
Menu_Type;

static Menu_Type Root_Menu [] =
{
     {"Color Test", color_test},
     {NULL, NULL}
};

Menu_Type *Current_Menu = Root_Menu;


static void write_centered_string (char *s, int row)
{
   unsigned int len;
   int col;
   
   if (s == NULL)
     return;
   
   len = strlen (s);
   
   /* Want 2 * col + len == SLtt_Screen_Rows */
   if (len >= (unsigned int) SLtt_Screen_Cols) col = 0;
   else col = (SLtt_Screen_Cols - (int)len) / 2;
   
   SLsmg_gotorc (row, col);
   SLsmg_write_string (s);
}

static void pre_test (char *title)
{
   SLsig_block_signals ();
   SLsmg_cls ();
   write_centered_string (title, 0);
}

static void post_test (void)
{
   write_centered_string ("Press any key to return.", SLtt_Screen_Rows - 1);
   SLsmg_refresh ();
   SLsig_unblock_signals ();
   (void) SLkp_getkey ();
}

/* Various tests */

#define NUM_COLORS 16
static char *Colors [NUM_COLORS] =
{
   "black/default",
   "red",
   "green",
   "brown",
   "blue",
   "magenta",
   "cyan",
   "lightgray",
   "gray",
   "brightred",
   "brightgreen",
   "yellow",
   "brightblue",
   "brightmagenta",
   "brightcyan",
   "white"
};

static void init_colors (void)
{
   int i;
   char *fg, *bg;

   fg = "black";
//   SLtt_set_color (1, NULL, fg, "default");
   SLtt_set_color (0, NULL, "white", "blue");

//   SLtt_set_color (1+NUM_COLORS, NULL, "default", "fg");
   SLtt_set_color (0, NULL, "white", "blue");

   for (i = 1; i < NUM_COLORS; i++)
     {
	bg = Colors[i];
	SLtt_set_color (i + 1, NULL, fg, bg);
	SLtt_set_color (i + 1 + NUM_COLORS, NULL, bg, fg);
     }
}




static int check_color_support (void)
{
   if (SLtt_Use_Ansi_Colors)
     return 0;
   
   pre_test ("Your terminal does not provide color support.");
   post_test ();
   return -1;
}




static void color_test (void)
{
   int color;
   int row;
   
   if (-1 == check_color_support ())
     return;

   pre_test ("Color Test");
   
   row = 1;

   color = 0;
   while (row < SLtt_Screen_Rows - 1)
     {
	color = color % NUM_COLORS;

	SLsmg_gotorc (row, 0);
	SLsmg_set_color (color+1+NUM_COLORS);
	SLsmg_write_string (Colors[color]);
	color++;
	SLsmg_set_color (color);
	SLsmg_erase_eol ();
	row++;
     }
   
   SLsmg_set_color (0);
   post_test ();
}



typedef struct 
{
   char *name;
   unsigned char value;
}
Draw_Symbols_Type;




void handle_winch (const int sig) 
{

	bool resize_enlarge ;
	int old_lines=LINES;
	resize_enlarge = FALSE;

	signal(sig, SIG_IGN);
	window_changed_size = ttysize();
	//window_changed_size = 1;
	signal(sig, handle_winch);
	

	if(old_lines<LINES) resize_enlarge=TRUE;

	// ��� ������� ���������� ����������� topfile � bottomfile
	// ���������� 
	if(!resize_enlarge) 
	{
		// 1-� ������� - ������ �������� � ���� ���������
		if(left.topfile+height() > left.curfile) 
			left.bottomfile = left.topfile + height();
		// 2-� ������� - ������ ����� �� ������� ������� �����
		else 
		{
			left.bottomfile = left.curfile+1;
			left.topfile = left.bottomfile-height();
		}
		
		if(right.topfile+2*height() > right.curfile) 
			right.bottomfile = right.topfile + 2*height();
		else 
		{
			right.bottomfile = right.curfile+1;
			right.topfile = right.bottomfile-2*height();
		}
	}
	// ���������� 
	else
	{
		if(left.bottomfile-height() > 0)
			left.topfile = left.bottomfile-height();
		else 
			left.topfile=0;
		left.bottomfile = left.topfile + height();
		if (left.bottomfile > left.num) left.bottomfile = left.num;

		if(right.bottomfile-2*height() > 0)
			right.topfile = right.bottomfile-2*height();
		else 
			right.topfile=0;
		right.bottomfile = right.topfile + 2*height();
		if (right.bottomfile > right.num) right.bottomfile = right.num;
	}


	if(EDIT)
	{
//		clear_entire_screen();
//		keep_cursor_on_screen(cur_buffer);
//		reset_window();

		clear_entire_screen();
		ttysize();
		reset_window();

	}
	else 
	{
		left.curfile=left.topfile=right.curfile=right.topfile=0;
		DisplayPanels();
	}




}
