/*
**  Copyright 1998-2003 University of Illinois Board of Trustees
**  Copyright 1998-2003 Mark D. Roth
**  All rights reserved.
**
**  libtar.c - demo driver program for libtar
**
**  Mark D. Roth <roth@uiuc.edu>
**  Campus Information Technologies and Educational Services
**  University of Illinois at Urbana-Champaign
*/

#include "config.h"
#include "libtar.h"
#include <syslog.h>

#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/param.h>
# include <string.h>
# include <unistd.h>
# include <signal.h>
# include "../zlib/zlib.h"

#include "compat.h"


char *progname;
int verbose = 0;
int use_gnu = 0;
int use_zlib = 0;

int gzopen_frontend(char *pathname, int oflags, int mode)
{
	char *gzoflags;
	gzFile gzf;
	int fd;

	switch (oflags & O_ACCMODE)
	{
	case O_WRONLY:
		gzoflags = "wb";
		break;
	case O_RDONLY:
		gzoflags = "rb";
		break;
	default:
	case O_RDWR:
		errno = EINVAL;
		return -1;
	}

	fd = open(pathname, oflags, mode);
	if (fd == -1)
		return -1;

	if ((oflags & O_CREAT) && fchmod(fd, mode))
		return -1;

	gzf = gzdopen(fd, gzoflags);
	if (!gzf)
	{
		errno = ENOMEM;
		return -1;
	}

	return (int)gzf;
}

tartype_t gztype = { (openfunc_t) gzopen_frontend, (closefunc_t) gzclose,
	(readfunc_t) gzread, (writefunc_t) gzwrite
};

//#endif /* HAVE_LIBZ */


int
create(char *tarfile, char *rootdir, libtar_list_t *l)
{
	TAR *t;
	char *pathname;
	char buf[MAXPATHLEN];
	libtar_listptr_t lp;

	if (tar_open(&t, tarfile,
		     (use_zlib ? &gztype : NULL),
		     O_WRONLY | O_CREAT, 0644,
		     (verbose ? TAR_VERBOSE : 0)
		     | (use_gnu ? TAR_GNU : 0)) == -1)
	{
		fprintf(stderr, "tar_open(): %s\n", strerror(errno));
		return -1;
	}

	libtar_listptr_reset(&lp);
	while (libtar_list_next(l, &lp) != 0)
	{
		pathname = (char *)libtar_listptr_data(&lp);
		if (pathname[0] != '/' && rootdir != NULL)
			snprintf(buf, sizeof(buf), "%s/%s", rootdir, pathname);
		else
			strlcpy(buf, pathname, sizeof(buf));
		if (tar_append_tree(t, buf, pathname) != 0)
		{
			fprintf(stderr,
				"tar_append_tree(\"%s\", \"%s\"): %s\n", buf,
				pathname, strerror(errno));
			tar_close(t);
			return -1;
		}
	}

	if (tar_append_eof(t) != 0)
	{
		fprintf(stderr, "tar_append_eof(): %s\n", strerror(errno));
		tar_close(t);
		return -1;
	}

	if (tar_close(t) != 0)
	{
		fprintf(stderr, "tar_close(): %s\n", strerror(errno));
		return -1;
	}

	return 0;
}


int list(char *tarfile)
{
	TAR *t;
	int i;


	if (tar_open(&t, 
								tarfile,
		      			&gztype ,
		     				O_RDONLY, 
								0,
		     				TAR_GNU ) == -1)
	{
		//fprintf(stderr, "tar_open(): %s\n", strerror(errno));
		return -1;
	}

	while ((i = th_read(t)) == 0)
	{
		th_print_long_ls(t);

		if (TH_ISREG(t) && tar_skip_regfile(t) != 0)
		{
			//fprintf(stderr, "tar_skip_regfile(): %s\n",	strerror(errno));
			//return -1;
		}
	}

  gzseek((gzFile) t->fd, 0, SEEK_CUR);

	if (tar_close(t) != 0)
	{
		//fprintf(stderr, "tar_close(): %s\n", strerror(errno));
		return -1;
	}


	return 0;
}


int ice_extract(char *tarfile, char *target, char * path , char * file, int type)
{
	TAR *t;


	if (tar_open(&t, 
								tarfile,
							 &gztype ,
		     				O_RDONLY, 
								0,
		     				TAR_GNU ) == -1)
	{	}

	int result = 0 ;
	if(type==2)
	 result =  ice_tar_extract_file(t, target,path,file);
	else
	 result =  ice_tar_extract_dir(t, target,path,file,type);

	if (tar_close(t) != 0)
	{
	}

	return 0;
}


int ice_tar(char * tar_file , char * target, char * path , char * file, int type)
{
	char *tarfile = NULL;
	//char *rootdir = NULL;
	int c;
	int mode = 0;
	libtar_list_t *l;

	int i ;


	if(type==1) // list
	{
		int result = list(tar_file);
		return result ;
	}
	else
	{
		return ice_extract(tar_file, target, path , file, type);
	}

	return -2;
}


