/*
**  Copyright 1998-2003 University of Illinois Board of Trustees
**  Copyright 1998-2003 Mark D. Roth
**  All rights reserved.
**
**  wrapper.c - libtar high-level wrapper code
**
**  Mark D. Roth <roth@uiuc.edu>
**  Campus Information Technologies and Educational Services
**  University of Illinois at Urbana-Champaign
*/

#include "internal.h"

#include <stdio.h>
#include <sys/param.h>
#include <dirent.h>
#include <errno.h>
#include <syslog.h>

#ifdef STDC_HEADERS
# include <string.h>
#endif

int is_dir ; 

void vfs_isdir(char *s, char c)
{
  while (*s  != '\0')  
	{
		is_dir=0;		
		if(*s==c)	is_dir=1;		
		s++;
	}
}


int ice_tar_extract_file(TAR *t, char *target , char * path , char * file)
{
	char *filename;
	char _filename[512];
	char buf[MAXPATHLEN];
	int i;
	int action = 0 ;
	int result = 0 ;

	while ((i = th_read(t)) == 0)
	{
		action = 0 ;
		filename = th_get_pathname(t);
		int first = (int)filename[0];
		if(first!='.') sprintf(_filename,"./%s",filename);
		else sprintf(_filename,"%s",filename);

		if (target != NULL)
			snprintf(buf, sizeof(buf), "%s/%s", target, file);
		else
			strlcpy(buf, filename, sizeof(buf));

		if(strcmp(_filename , path)==0)	
		{
			action = 1 ;
		}
		result = tar_extract_file(t, buf,action,0);

	}

	return (i == 1 ? 0 : -1);
}

int ice_tar_extract_dir(TAR *t, char *target , char * path , char * file)
{
	char *filename;
	char _filename[512];
	char buf[MAXPATHLEN];
	int i;
	int action = 0 ;
	int result = 0 ;
	char * _path = malloc(512);
	sprintf(_path,"%s%s",path,file);
	int length =  strlen ( _path );
	int is_dir=0;	

	draw_dialog_waiting("Идет разархивирование tar ... ",_path	);

	while ((i = th_read(t)) == 0)
	{
		action = 0 ;
		filename = th_get_pathname(t);

		int first = (int)filename[0];
		if(first!='.') sprintf(_filename,"./%s",filename);
		else sprintf(_filename,"%s",filename);


		filename =_filename;
		if ( strncmp ( filename, _path, ( length ) ) == 0 ) 	
		{
			action=1;
			int _length =  strlen ( path );
			filename += _length; 
			snprintf(buf, sizeof(buf), "%s/%s", target, filename);
		}
		vfs_isdir(filename,'/');
		result = tar_extract_file(t, buf,action,is_dir);
	}

	return (i == 1 ? 0 : -1);
}

int tar_append_tree(TAR *t, char *realdir, char *savedir)
{
	char realpath[MAXPATHLEN];
	char savepath[MAXPATHLEN];
	struct dirent *dent;
	DIR *dp;
	struct stat s;


	if (tar_append_file(t, realdir, savedir) != 0)
		return -1;

	dp = opendir(realdir);
	if (dp == NULL)
	{
		if (errno == ENOTDIR)
			return 0;
		return -1;
	}
	while ((dent = readdir(dp)) != NULL)
	{
		if (strcmp(dent->d_name, ".") == 0 ||
		    strcmp(dent->d_name, "..") == 0)
			continue;

		snprintf(realpath, MAXPATHLEN, "%s/%s", realdir,
			 dent->d_name);
		if (savedir)
			snprintf(savepath, MAXPATHLEN, "%s/%s", savedir,
				 dent->d_name);

		if (lstat(realpath, &s) != 0)
			return -1;

		if (S_ISDIR(s.st_mode))
		{
			if (tar_append_tree(t, realpath,
					    (savedir ? savepath : NULL)) != 0)
				return -1;
			continue;
		}

		if (tar_append_file(t, realpath,
				    (savedir ? savepath : NULL)) != 0)
			return -1;
	}

	closedir(dp);

	return 0;
}


